@@ -117,6 +117,7 @@ enum {
 static void popup_menu_cb (gpointer data, guint action, GtkWidget *widget);
 static gint save_uri_cb (BonoboPersistFile *pf, const CORBA_char *text_uri,
 			 CORBA_Environment *ev, void *closure);
+static gboolean save_uri (EogImageView *view, const char *text_uri, GdkPixbufFormat *format);
 
 
 static guint signals[LAST_SIGNAL] = { 0 };
@@ -316,6 +317,7 @@ verb_SaveAs_cb (BonoboUIComponent *uic, gpointer user_data,
 	GtkWidget        *dlg;
 	int              response;
 	gchar            *filename = NULL;
+	GdkPixbufFormat  *format = NULL;
 
 	g_return_if_fail (EOG_IS_IMAGE_VIEW (user_data));
 
@@ -323,21 +325,20 @@ verb_SaveAs_cb (BonoboUIComponent *uic, gpointer user_data,
 
 	if (image_view->priv->image == NULL) return;
 
-	dlg = eog_file_selection_new (EOG_FILE_SELECTION_SAVE);
+	dlg = eog_file_selection_new (GTK_FILE_CHOOSER_ACTION_SAVE);
 	gtk_widget_show_all (dlg);
 
 	response = gtk_dialog_run (GTK_DIALOG (dlg));
 
 	if (response == GTK_RESPONSE_OK) {
-		filename = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (dlg)));
+		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dlg));
+		format = eog_file_selection_get_format (EOG_FILE_SELECTION (dlg));
 	}
 
 	gtk_widget_destroy (dlg);
 
-	
-
 	if (response == GTK_RESPONSE_OK) {
-		save_uri_cb (NULL, filename, NULL, image_view);
+		save_uri (image_view, filename, format);
 	}
 
 	if (filename != NULL) {
@@ -503,12 +504,25 @@ eog_image_view_get_prop (BonoboPropertyBag *bag,
 		break;
 	}
 	case PROP_WINDOW_TITLE: {
+		char *caption;
+
 		g_assert (arg->_type == BONOBO_ARG_STRING);
 
-		if (priv->image != NULL) {
-			BONOBO_ARG_SET_STRING (arg, eog_image_get_caption (priv->image));
+		caption = g_strdup (eog_image_get_caption (priv->image)); 
+
+		if (eog_image_is_modified (priv->image)) {
+			char *tmp;
+
+			tmp = g_strconcat (caption, "*", NULL);
+			g_free (caption);
+			caption = tmp;
 		}
 
+		if (priv->image != NULL) {
+			BONOBO_ARG_SET_STRING (arg, caption);
+		}
+		g_free (caption);
+
 		break;
 	}
 	case PROP_WINDOW_STATUS: {
@@ -977,6 +991,14 @@ image_changed_cb (EogImage *img, gpointer data)
 					      "window/status",
 					      arg, NULL);
 	bonobo_arg_release (arg);
+
+	arg = bonobo_arg_new (BONOBO_ARG_STRING);
+	eog_image_view_get_prop (NULL, arg, PROP_WINDOW_TITLE, NULL, view);
+
+	bonobo_event_source_notify_listeners (view->priv->property_bag->es,
+					      "window/title",
+					      arg, NULL);
+	bonobo_arg_release (arg);
 }
 
 static void
@@ -1081,22 +1103,26 @@ static gint
 save_uri_cb (BonoboPersistFile *pf, const CORBA_char *text_uri,
 	     CORBA_Environment *ev, void *closure)
 {
-	EogImageView *view;
+	return save_uri (EOG_IMAGE_VIEW (closure), text_uri, NULL);
+}
+
+static gboolean
+save_uri (EogImageView *view, const char *text_uri, GdkPixbufFormat *format)
+{
 	GnomeVFSURI *uri;
 	GError *error = NULL;
 	GtkWidget *dialog;
 	gboolean result;
 
-	g_return_val_if_fail (EOG_IS_IMAGE_VIEW (closure), 1);
-	g_return_val_if_fail (text_uri != NULL, 1);
+	g_return_val_if_fail (EOG_IS_IMAGE_VIEW (view), FALSE);
+	g_return_val_if_fail (text_uri != NULL, FALSE);
 
-	view = EOG_IMAGE_VIEW (closure);
 	if (view->priv->image == NULL) return FALSE;
 
 	/* FIXME: what kind of escaping do we need here? */
 	uri = gnome_vfs_uri_new (text_uri);
 
-	result = eog_image_save (view->priv->image, uri, &error);
+	result = eog_image_save (view->priv->image, uri, format, &error);
 
 	if (result) {
 		dialog = eog_hig_dialog_new (GTK_STOCK_DIALOG_INFO,
